﻿namespace HeuristicLab.Algorithms.PushGP
{
    public enum OpCode : byte
    {
        #region Boolean
        /// <summary>
        /// BOOLEAN.=:
        /// Pushes TRUE if the top two BOOLEANs are equal, or FALSE otherwise
        /// </summary>
        BooleanEquals,

        /// <summary>
        /// BOOLEAN.AND:
        /// Pushes the logical AND of the top two BOOLEANs
        /// </summary>
        BooleanAnd,

        /// <summary>
        /// BOOLEAN.DEFINE:
        /// Defines the name on top of the NAME stack as an inclassion that will push the top item of the BOOLEAN stack onto the EXEC stack.
        /// </summary>
        BooleanDefine,

        /// <summary>
        /// BOOLEAN.DUP:
        /// Duplicatelicates the top item on the BOOLEAN stack. Does not pop its argument (which, if it did, would negate the effect of the duplication!)
        /// </summary>
        BooleanDuplicate,

        /// <summary>
        /// BOOLEAN.FLUSH:
        /// Empties the BOOLEAN stack.
        /// </summary>
        BooleanFlush,

        /// <summary>
        /// BOOLEAN.FROMFLOAT:
        /// Pushes FALSE if the top FLOAT is 0.0, or TRUE otherwise.
        /// </summary>
        BooleanFromFloat,

        /// <summary>
        /// BOOLEAN.FROMINTEGER:
        /// Pushes FALSE if the top INTEGER is 0, or TRUE otherwise.
        /// </summary>
        BooleanFromInteger,

        /// <summary>
        /// BOOLEAN.NOT:
        /// Pushes the logical NOT of the top BOOLEAN.
        /// </summary>
        BooleanNot,

        /// <summary>
        /// BOOLEAN.OR:
        /// Pushes the logical OR of the top two BOOLEANs.
        /// </summary>
        BooleanOr,

        /// <summary>
        /// BOOLEAN.POP:
        /// Pops the BOOLEAN stack.
        /// </summary>
        BooleanPop,

        /// <summary>
        /// Pushes a boolean literal to the BOOLEAN stack.
        /// </summary>
        BooleanPush,

        /// <summary>
        /// BOOLEAN.RAND:
        /// Pushes a random BOOLEAN.
        /// </summary>
        BooleanRand,

        /// <summary>
        /// BOOLEAN.ROT
        ///  Rotates the top three items on the BOOLEAN stack, pulling the third item out and pushing it on top. This is equivalent to "2 BOOLEAN.YANK".
        /// </summary>
        BooleanRot,

        /// <summary>
        /// BOOLEAN.SHOVE:
        /// Inserts the top BOOLEAN "deep" in the stack, at the position indexed by the top INTEGER.
        /// </summary>
        BooleanShove,

        /// <summary>
        /// BOOLEAN.STACKDEPTH
        /// Pushes the stack depth onto the INTEGER stack.
        /// </summary>
        BooleanStackDepth,

        /// <summary>
        /// BOOLEAN.SWAP:
        /// Swaps the top two BOOLEANs.
        /// </summary>
        BooleanSwap,

        /// <summary>
        /// BOOLEAN.YANK:
        /// Removes an indexed item from "deep" in the stack and pushes it on top of the stack. The index is taken from the INTEGER stack.
        /// </summary>
        BooleanYank,

        /// <summary>
        /// BOOLEAN.YANKDUP:
        /// Pushes a copy of an indexed item "deep" in the stack onto the top of the stack, without removing the deep item. The index is taken from the INTEGER stack.
        /// </summary>
        BooleanYankDuplicate,
        #endregion Boolean

        #region Code
        /// <summary>
        /// CODE.=:
        /// Pushes TRUE if the top two pieces of CODE are equal, or FALSE otherwise.
        /// </summary>
        CodeEquals,

        CodePush,

        /// <summary>
        /// CODE.APPEND:
        /// Pushes the result of appending the top two pieces of code. If one of the pieces of code is a single inclassion or literal (that is, something not surrounded by parentheses) then it is surrounded by parentheses first.
        /// </summary>
        CodeAppend,

        /// <summary>
        /// CODE.ATOM:
        /// Pushes TRUE onto the BOOLEAN stack if the top piece of code is a single inclassion or a literal, and FALSE otherwise (that is, if it is something surrounded by parentheses).
        /// </summary>
        CodeAtom,

        /// <summary>
        /// CODE.CAR
        /// Pushes the first item of the list on top of the CODE stack. For example, if the top piece of code is "( A B )" then this pushes "A" (after popping the argument). If the code on top of the stack is not a list then this has no effect. The name derives from the similar Lisp function; a more generic name would be "FIRST".
        /// </summary>
        CodeCar,

        /// <summary>
        /// CODE.CDR:
        /// Pushes a version of the list from the top of the CODE stack without its first element. For example, if the top piece of code is "( A B )" then this pushes "( B )" (after popping the argument). If the code on top of the stack is not a list then this pushes the empty list ("( )"). The name derives from the similar Lisp function; a more generic name would be "REST".
        /// </summary>
        CodeCdr,

        /// <summary>
        /// CODE.CONS:
        /// Pushes the result of "consing" (in the Lisp sense) the second stack item onto the first stack item (which is coerced to a list if necessary). For example, if the top piece of code is "( A B )" and the second piece of code is "X" then this pushes "( X A B )" (after popping the argument).
        /// </summary>
        CodeCons,

        /// <summary>
        /// CODE:CONTAINS:
        /// Pushes the "container" of the second CODE stack item within the first CODE stack item onto the CODE stack. If second item contains the first anywhere (i.e. in any nested list) then the container is the smallest sub-list that contains but is not equal to the first instance. For example, if the top piece of code is "( B ( C ( A ) ) ( D ( A ) ) )" and the second piece of code is "( A )" then this pushes ( C ( A ) ). Pushes an empty list if there is no such container.
        /// </summary>
        CodeContainer,

        /// <summary>
        /// CODE.CONTAINS:
        /// Pushes TRUE on the BOOLEAN stack if the second CODE stack item contains the first CODE stack item anywhere (e.g. in a sub-list).
        /// </summary>
        CodeContains,

        /// <summary>
        /// CODE.DEFINE:
        /// Defines the name on top of the NAME stack as an inclassion that will push the top item of the CODE stack onto the EXEC stack.
        /// </summary>
        CodeDefine,

        /// <summary>
        /// CODE.DEFINITION:
        /// Pushes the definition associated with the top NAME on the NAME stack (if any) onto the CODE stack. This extracts the definition for inspection/manipulation, rather than for immediate execution (although it may then be executed with a call to CODE.DO or a similar inclassion).
        /// </summary>
        CodeDefinition,

        /// <summary>
        /// CODE.DESCREPANCY:
        /// Pushes a measure of the discrepancy between the top two CODE stack items onto the INTEGER stack. This will be zero if the top two items are equivalent, and will be higher the 'more different' the items are from one another. The calculation is as follows:
        /// 1. Conclass a list of all of the unique items in both of the lists (where uniqueness is determined by equalp). Sub-lists and atoms all count as items.
        /// 2. Initialize the result to zero.
        /// 3. For each unique item increment the result by the difference between the number of occurrences of the item in the two pieces of code.
        /// 4. Push the result.
        /// </summary>
        CodeDiscrepancy,

        /// <summary>
        /// CODE.DO:
        /// Recursively invokes the interpreter on the program on top of the CODE stack. After evaluation the CODE stack is popped; normally this pops the program that was just executed, but if the expression itself manipulates the stack then this final pop may end up popping something else.
        /// </summary>
        CodeDo,

        /// <summary>
        /// CODE.DO*:
        /// Like CODE.DO but pops the stack before, rather than after, the recursive execution.
        /// </summary>
        CodeDoX,

        /// <summary>
        /// CODE.DO*COUNT:
        /// An iteration inclassion that performs a loop (the body of which is taken from the CODE stack) the number of times indicated by the INTEGER argument, pushing an index (which runs from zero to one less than the number of iterations) onto the INTEGER stack prior to each execution of the loop body. This should be implemented as a macro that expands into a call to CODE.DO*RANGE. CODE.DO*COUNT takes a single INTEGER argument (the number of times that the loop will be executed) and a single CODE argument (the body of the loop). If the provided INTEGER argument is negative or zero then this becomes a NOOP. Otherwise it expands into:
        /// ( 0 '1 - IntegerArg' CODE.QUOTE <CodeArg> CODE.DO*RANGE )
        /// </summary>
        CodeDoXCount,

        /// <summary>
        /// CODE.DO*RANGE:
        /// An iteration inclassion that executes the top item on the CODE stack a number of times that depends on the top two integers, while also pushing the loop counter onto the INTEGER stack for possible access during the execution of the body of the loop. The top integer is the "destination index" and the second integer is the "current index." First the code and the integer arguments are saved locally and popped. Then the integers are compared. If the integers are equal then the current index is pushed onto the INTEGER stack and the code (which is the "body" of the loop) is pushed onto the EXEC stack for subsequent execution. If the integers are not equal then the current index will still be pushed onto the INTEGER stack but two items will be pushed onto the EXEC stack -- first a recursive call to CODE.DO*RANGE (with the same code and destination index, but with a current index that has been either incremented or decremented by 1 to be closer to the destination index) and then the body code. Note that the range is inclusive of both endpoints; a call with integer arguments 3 and 5 will cause its body to be executed 3 times, with the loop counter having the values 3, 4, and 5. Note also that one can specify a loop that "counts down" by providing a destination index that is less than the specified current index.
        /// </summary>
        CodeDoXRange,

        /// <summary>
        /// CODE.DO*TIMES:
        /// Like CODE.DO*COUNT but does not push the loop counter. This should be implemented as a macro that expands into CODE.DO*RANGE, similarly to the implementation of CODE.DO*COUNT, except that a call to INTEGER.POP should be tacked on to the front of the loop body code in the call to CODE.DO*RANGE. This call to INTEGER.POP will remove the loop counter, which will have been pushed by CODE.DO*RANGE, prior to the execution of the loop body.
        /// </summary>
        CodeDoXTimes,

        /// <summary>
        /// CODE.DUP:
        /// Duplicatelicates the top item on the CODE stack. Does not pop its argument (which, if it did, would negate the effect of the duplication!).
        /// </summary>
        CodeDuplicate,

        /// <summary>
        /// CODE.EXTRACT:
        /// Pushes the sub-expression of the top item of the CODE stack that is indexed by the top item of the INTEGER stack. The indexing here counts "points," where each parenthesized expression and each literal/inclassion is considered a point, and it proceeds in depth first order. The entire piece of code is at index 0; if it is a list then the first item in the list is at index 1, etc. The integer used as the index is taken modulo the number of points in the overall expression (and its absolute value is taken in case it is negative) to ensure that it is within the meaningful range.
        /// </summary>
        CodeExtract,

        /// <summary>
        /// CODE.FLUSH:
        /// Empties the CODE stack.
        /// </summary>
        CodeFlush,

        /// <summary>
        /// CODE.FROMBOOLEAN:
        /// Pops the BOOLEAN stack and pushes the popped item (TRUE or FALSE) onto the CODE stack.
        /// </summary>
        CodeFromBoolean,

        /// <summary>
        /// CODE.FROMFLOAT:
        /// Pops the FLOAT stack and pushes the popped item onto the CODE stack.
        /// </summary>
        CodeFromFloat,

        /// <summary>
        /// CODE.FROMINTEGER:
        /// Pops the INTEGER stack and pushes the popped integer onto the CODE stack.
        /// </summary>
        CodeFromInteger,

        /// <summary>
        /// CODE.FROMNAME:
        /// Pops the NAME stack and pushes the popped item onto the CODE stack.
        /// </summary>
        CodeFromName,

        /// <summary>
        /// CODE.IF:
        /// If the top item of the BOOLEAN stack is TRUE this recursively executes the second item of the CODE stack; otherwise it recursively executes the first item of the CODE stack. Either way both elements of the CODE stack (and the BOOLEAN value upon which the decision was made) are popped.
        /// </summary>
        CodeIf,

        /// <summary>
        /// CODE.INSTRUCTIONS:
        /// Pushes a list of all active instructions in the interpreter's current configuration.
        /// </summary>
        CodeInstructions,

        /// <summary>
        /// CODE.INSERT:
        /// Pushes the result of inserting the second item of the CODE stack into the first item, at the position indexed by the top item of the INTEGER stack (and replacing whatever was there formerly). The indexing is computed as in CODE.EXTRACT.
        /// </summary>
        CodeInsert,

        /// <summary>
        /// CODE.INSTRUCTIONS:
        /// Pushes a list of all active inclassions in the interpreter's current configuration.
        /// </summary>
        CodeInclassions,

        /// <summary>
        /// CODE.LENGTH:
        /// Pushes the length of the top item on the CODE stack onto the INTEGER stack. If the top item is not a list then this pushes a 1. If the top item is a list then this pushes the number of items in the top level of the list; that is, nested lists contribute only 1 to this count, no matter what they contain.
        /// </summary>
        CodeLength,

        /// <summary>
        /// CODE.LIST: Pushes a list of the top two items of the CODE stack onto the CODE stack.
        /// </summary>
        CodeList,

        /// <summary>
        /// CODE.MEMBER:
        /// Pushes TRUE onto the BOOLEAN stack if the second item of the CODE stack is a member of the first item (which is coerced to a list if necessary). Pushes FALSE onto the BOOLEAN stack otherwise.
        /// </summary>
        CodeMember,

        /// <summary>
        /// CODE.NOOP: Does nothing.
        /// </summary>
        CodeNoop,

        /// <summary>
        /// CODE:NTH:
        /// Pushes the nth element of the expression on top of the CODE stack (which is coerced to a list first if necessary). If the expression is an empty list then the result is an empty list. N is taken from the INTEGER stack and is taken modulo the length of the expression into which it is indexing.
        /// </summary>
        CodeNth,

        /// <summary>
        /// CODE.NTHCDR:
        /// Pushes the nth "CDR" (in the Lisp sense) of the expression on top of the CODE stack (which is coerced to a list first if necessary). If the expression is an empty list then the result is an empty list. N is taken from the INTEGER stack and is taken modulo the length of the expression into which it is indexing. A "CDR" of a list is the list without its first element.
        /// </summary>
        CodeNthCdr,

        /// <summary>
        /// CODE.NULL: Pushes TRUE onto the BOOLEAN stack if the top item of the CODE stack is an empty list, or FALSE otherwise.
        /// </summary>
        CodeNull,

        /// <summary>
        /// CODE.POSITION:
        /// Pushes onto the INTEGER stack the position of the second item on the CODE stack within the first item (which is coerced to a list if necessary). Pushes -1 if no match is found.
        /// </summary>
        CodePosition,

        /// <summary>
        /// CODE.QUOTE:
        /// Specifies that the next expression submitted for execution will instead be pushed literally onto the CODE stack. This can be implemented by moving the top item on the EXEC stack onto the CODE stack.
        /// </summary>
        CodeQuote,

        /// <summary>
        /// CODE.RAND: Pushes a newly-generated random program onto the CODE stack. The limit for the size of the expression is taken from the INTEGER stack; to ensure that it is in the appropriate range this is taken modulo the value of the MAX-POINTS-IN-RANDOM-EXPRESSIONS parameter and the absolute value of the result is used.
        /// </summary>
        CodeRand,

        /// <summary>
        /// CODE.ROT:
        /// Rotates the top three items on the CODE stack, pulling the third item out and pushing it on top. This is equivalent to "2 CODE.YANK".
        /// </summary>
        CodeRot,

        /// <summary>
        /// CODE.SHOVE:
        /// Inserts the top piece of CODE "deep" in the stack, at the position indexed by the top INTEGER.
        /// </summary>
        CodeShove,

        /// <summary>
        /// CODE.SIZE: Pushes the number of "points" in the top piece of CODE onto the INTEGER stack. Each inclassion, literal, and pair of parentheses counts as a point.
        /// </summary>
        CodeSize,

        /// <summary>
        /// CODE.STACKDEPTH: Pushes the stack depth onto the INTEGER stack.
        /// </summary>
        CodeStackDepth,

        /// <summary>
        /// CODE.SUBST: Pushes the result of substituting the third item on the code stack for the second item in the first item. As of this writing this is implemented only in the Lisp implementation, within which it relies on the Lisp "subst" function. As such, there are several problematic possibilities; for example "dotted-lists" can result in certain cases with empty-list arguments. If any of these problematic possibilities occurs the stack is left unchanged.
        /// </summary>
        CodeSubst,

        /// <summary>
        /// CODE.SWAP:
        /// Swaps the top two pieces of CODE.
        /// </summary>
        CodeSwap,

        /// <summary>
        /// CODE.YANK:
        /// Removes an indexed item from "deep" in the stack and pushes it on top of the stack. The index is taken from the INTEGER stack.
        /// </summary>
        CodeYank,

        /// <summary>
        /// CODE.YANKDUP:
        /// Pushes a copy of an indexed item "deep" in the stack onto the top of the stack, without removing the deep item. The index is taken from the INTEGER stack.
        /// </summary>
        CodeYankDuplicate,
        #endregion Code

        #region Exec
        ExecExpand,

        ExecPush,

        /// <summary>
        /// EXEC.=:
        /// Pushes TRUE if the top two items on the EXEC stack are equal, or FALSE otherwise.
        /// </summary>
        ExecEquals,

        /// <summary>
        /// EXEC.DEFINE:
        /// Defines the name on top of the NAME stack as an inclassion that will push the top item of the EXEC stack back onto the EXEC stack.
        /// </summary>
        ExecDefine,

        /// <summary>
        /// EXEC.DO*COUNT:
        /// An iteration inclassion that performs a loop (the body of which is taken from the EXEC stack) the number of times indicated by the INTEGER argument, pushing an index (which runs from zero to one less than the number of iterations) onto the INTEGER stack prior to each execution of the loop body. This is similar to CODE.DO*COUNT except that it takes its code argument from the EXEC stack. This should be implemented as a macro that expands into a call to EXEC.DO*RANGE. EXEC.DO*COUNT takes a single INTEGER argument (the number of times that the loop will be executed) and a single EXEC argument (the body of the loop). If the provided INTEGER argument is negative or zero then this becomes a NOOP. Otherwise it expands into:
        /// ( 0 <1 - IntegerArg> EXEC.DO*RANGE <ExecArg> )
        /// </summary>
        ExecDoXCount,

        /// <summary>
        /// EXEC.DO*RANGE:
        /// An iteration inclassion that executes the top item on the EXEC stack a number of times that depends on the top two integers, while also pushing the loop counter onto the INTEGER stack for possible access during the execution of the body of the loop. This is similar to CODE.DO*COUNT except that it takes its code argument from the EXEC stack. The top integer is the "destination index" and the second integer is the "current index." First the code and the integer arguments are saved locally and popped. Then the integers are compared. If the integers are equal then the current index is pushed onto the INTEGER stack and the code (which is the "body" of the loop) is pushed onto the EXEC stack for subsequent execution. If the integers are not equal then the current index will still be pushed onto the INTEGER stack but two items will be pushed onto the EXEC stack -- first a recursive call to EXEC.DO*RANGE (with the same code and destination index, but with a current index that has been either incremented or decremented by 1 to be closer to the destination index) and then the body code. Note that the range is inclusive of both endpoints; a call with integer arguments 3 and 5 will cause its body to be executed 3 times, with the loop counter having the values 3, 4, and 5. Note also that one can specify a loop that "counts down" by providing a destination index that is less than the specified current index.
        /// </summary>
        ExecDoXRange,

        /// <summary>
        /// EXEC.DO*TIMES:
        /// Like EXEC.DO*COUNT but does not push the loop counter. This should be implemented as a macro that expands into EXEC.DO*RANGE, similarly to the implementation of EXEC.DO*COUNT, except that a call to INTEGER.POP should be tacked on to the front of the loop body code in the call to EXEC.DO*RANGE. This call to INTEGER.POP will remove the loop counter, which will have been pushed by EXEC.DO*RANGE, prior to the execution of the loop body.
        /// </summary>
        ExecDoXTimes,

        /// <summary>
        /// EXEC.DUP:
        /// Duplicatelicates the top item on the EXEC stack. Does not pop its argument (which, if it did, would negate the effect of the duplication!). This may be thought of as a "DO TWICE" inclassion.
        /// </summary>
        ExecDuplicate,

        /// <summary>
        /// EXEC.FLUSH:
        /// Empties the EXEC stack. This may be thought of as a "HALT" inclassion.
        /// </summary>
        ExecFlush,

        /// <summary>
        /// EXEC.IF:
        /// If the top item of the BOOLEAN stack is TRUE then this removes the second item on the EXEC stack, leaving the first item to be executed. If it is false then it removes the first item, leaving the second to be executed. This is similar to CODE.IF except that it operates on the EXEC stack. This acts as a NOOP unless there are at least two items on the EXEC stack and one item on the BOOLEAN stack.
        /// </summary>
        ExecIf,

        /// <summary>
        /// EXEC.K:
        /// The Push implementation of the "K combinator". Removes the second item on the EXEC stack.
        /// </summary>
        ExecK,

        /// <summary>
        /// EXEC.POP: Pops the EXEC stack. This may be thought of as a "DONT" inclassion.
        /// </summary>
        ExecPop,

        /// <summary>
        /// EXEC.ROT:
        /// Rotates the top three items on the EXEC stack, pulling the third item out and pushing it on top. This is equivalent to "2 EXEC.YANK".
        /// </summary>
        ExecRot,

        /// <summary>
        /// EXEC.S:
        /// The Push implementation of the "S combinator". Pops 3 items from the EXEC stack, which we will call A, B, and C (with A being the first one popped). Then pushes a list containing B and C back onto the EXEC stack, followed by another instance of C, followed by another instance of A.
        /// </summary>
        ExecS,

        /// <summary>
        /// EXEC.SHOVE:
        /// Inserts the top EXEC item "deep" in the stack, at the position indexed by the top INTEGER. This may be thought of as a "DO LATER" inclassion.
        /// </summary>
        ExecShove,

        /// <summary>
        /// EXEC.STACKDEPTH:
        /// Pushes the stack depth onto the INTEGER stack.
        /// </summary>
        ExecStackDepth,

        /// <summary>
        /// EXEC.SWAP:
        /// Swaps the top two items on the EXEC stack.
        /// </summary>
        ExecSwap,

        /// <summary>
        /// EXEC.Y:
        /// The Push implementation of the "Y combinator". Inserts beneath the top item of the EXEC stack a new item of the form "( EXEC.Y <TopItem> )".
        /// </summary>
        ExecY,

        /// <summary>
        /// EXEC.YANK:
        /// Removes an indexed item from "deep" in the stack and pushes it on top of the stack. The index is taken from the INTEGER stack. This may be thought of as a "DO SOONER" inclassion.
        /// </summary>
        ExecYank,

        /// <summary>
        /// EXEC.YANKDUP:
        /// Pushes a copy of an indexed item "deep" in the stack onto the top of the stack, without removing the deep item. The index is taken from the INTEGER stack.
        /// </summary>
        ExecYankDuplicate,
        #endregion Exec

        #region Float
        /// <summary>
        /// FLOAT.%:
        /// Pushes the second stack item modulo the top stack item. If the top item is zero this acts as a NOOP. The modulus is computed as the remainder of the quotient, where the quotient has first been truncated toward negative infinity. (This is taken from the definition for the generic MOD function in Common Lisp, which is described for example at
        /// <see cref="http://www.lispworks.com/reference/HyperSpec/Body/f_mod_r.htm">HyperSpec</see>.)
        /// </summary>
        FloatModulo,

        /// <summary>
        /// FLOAT.*:
        /// Pushes the product of the top two items.
        /// </summary>
        FloatMultiply,

        /// <summary>
        /// FLOAT.+:
        /// Pushes the sum of the top two items.
        /// </summary>
        FloatAdd,

        /// <summary>
        /// FLOAT.-:
        /// Pushes the difference of the top two items; that is, the second item minus the top item.
        /// </summary>
        FloatSubtract,

        /// <summary>
        /// FLOAT./:
        /// Pushes the quotient of the top two items; that is, the second item divided by the top item. If the top item is zero this acts as a NOOP.
        /// </summary>
        FloatDivide,

        /// <summary>
        /// FLOAT.&lt;:
        /// Pushes TRUE onto the BOOLEAN stack if the second item is less than the top item, or FALSE otherwise.
        /// </summary>
        FloatSmallerThan,

        /// <summary>
        /// FLOAT.=:
        /// Pushes TRUE onto the BOOLEAN stack if the top two items are equal, or FALSE otherwise.
        /// </summary>
        FloatEquals,

        /// <summary>
        /// FLOAT.&gt;:
        /// Pushes TRUE onto the BOOLEAN stack if the second item is greater than the top item, or FALSE otherwise.
        /// </summary>
        FloatGreaterThan,

        /// <summary>
        /// FLOAT.COS:
        /// Pushes the cosine of the top item.
        /// </summary>
        FloatCos,

        /// <summary>
        /// FLOAT.DEFINE:
        /// Defines the name on top of the NAME stack as an inclassion that will push the top item of the FLOAT stack onto the EXEC stack.
        /// </summary>
        FloatDefine,

        /// <summary>
        /// FLOAT.DUP:
        /// Duplicatelicates the top item on the FLOAT stack. Does not pop its argument (which, if it did, would negate the effect of the duplication!).
        /// </summary>
        FloatDuplicate,

        /// <summary>
        /// FLOAT.FLUSH:
        /// Empties the FLOAT stack.
        /// </summary>
        FloatFlush,

        /// <summary>
        /// FLOAT.FROMBOOLEAN:
        /// Pushes 1.0 if the top BOOLEAN is TRUE, or 0.0 if the top BOOLEAN is FALSE.
        /// </summary>
        FloatFromBoolean,

        /// <summary>
        /// FLOAT.FROMINTEGER:
        /// Pushes a floating point version of the top INTEGER.
        /// </summary>
        FloatFromInteger,

        /// <summary>
        /// FLOAT.MAX:
        /// Pushes the maximum of the top two items.
        /// </summary>
        FloatMax,

        /// <summary>
        /// FLOAT.MIN:
        /// Pushes the minimum of the top two items.
        /// </summary>
        FloatMin,

        /// <summary>
        /// FLOAT.POP:
        /// Pops the FLOAT stack.
        /// </summary>
        FloatPop,

        /// <summary>
        /// Pushes a floating point value to the FLOAT stack.
        /// </summary>
        FloatPush,

        /// <summary>
        /// FLOAT.RAND:
        /// Pushes a newly generated random FLOAT that is greater than or equal to MIN-RANDOM-FLOAT and less than or equal to MAX-RANDOM-FLOAT.
        /// </summary>
        FloatRand,

        /// <summary>
        /// FLOAT.ROT:
        /// Rotates the top three items on the FLOAT stack, pulling the third item out and pushing it on top. This is equivalent to "2 FLOAT.YANK".
        /// </summary>
        FloatRot,

        /// <summary>
        /// FLOAT.SHOVE:
        /// Inserts the top FLOAT "deep" in the stack, at the position indexed by the top INTEGER.
        /// </summary>
        FloatShove,

        /// <summary>
        /// FLOAT.SIN:
        /// Pushes the sine of the top item.
        /// </summary>
        FloatSin,

        /// <summary>
        /// FLOAT.STACKDEPTH:
        /// Pushes the stack depth onto the INTEGER stack.
        /// </summary>
        FloatStackDepth,

        /// <summary>
        /// FLOAT.SWAP:
        /// Swaps the top two BOOLEANs.
        /// </summary>
        FloatSwap,

        /// <summary>
        /// FLOAT.TAN:
        /// Pushes the tangent of the top item.
        /// </summary>
        FloatTan,

        /// <summary>
        /// FLOAT.YANK:
        /// Removes an indexed item from "deep" in the stack and pushes it on top of the stack. The index is taken from the INTEGER stack.
        /// </summary>
        FloatYank,

        /// <summary>
        /// FLOAT.YANKDUP:
        /// Pushes a copy of an indexed item "deep" in the stack onto the top of the stack, without removing the deep item. The index is taken from the INTEGER stack.
        /// </summary>
        FloatYankDuplicate,

        #endregion Float

        #region Integer

        /// <summary>
        /// FLOAT.%:
        /// Pushes the second stack item modulo the top stack item. If the top item is zero this acts as a NOOP. The modulus is computed as the remainder of the quotient, where the quotient has first been truncated toward negative infinity. (This is taken from the definition for the generic MOD function in Common Lisp, which is described for example at
        /// <see cref="http://www.lispworks.com/reference/HyperSpec/Body/f_mod_r.htm">HyperSpec</see>.)
        /// </summary>
        IntegerModulo,

        /// <summary>
        /// INTEGER.*:
        /// Pushes the product of the top two items.
        /// </summary>
        IntegerMultiply,

        /// <summary>
        /// INTEGER.+:
        /// Pushes the sum of the top two items.
        /// </summary>
        IntegerAdd,

        /// <summary>
        /// INTEGER.-:
        /// Pushes the difference of the top two items; that is, the second item minus the top item.
        /// </summary>
        IntegerSubtract,

        /// <summary>
        /// INTEGER./:
        /// Pushes the quotient of the top two items; that is, the second item divided by the top item. If the top item is zero this acts as a NOOP.
        /// </summary>
        IntegerDivide,

        /// <summary>
        /// INTEGER.&lt;:
        /// Pushes TRUE onto the BOOLEAN stack if the second item is less than the top item, or FALSE otherwise.
        /// </summary>
        IntegerSmallerThan,

        /// <summary>
        /// INTEGER.=:
        /// Pushes TRUE onto the BOOLEAN stack if the top two items are equal, or FALSE otherwise.
        /// </summary>
        IntegerEquals,

        /// <summary>
        /// INTEGER.&gt;:
        /// Pushes TRUE onto the BOOLEAN stack if the second item is greater than the top item, or FALSE otherwise.
        /// </summary>
        IntegerGreaterThan,

        /// <summary>
        /// INTEGER.DEFINE:
        /// Defines the name on top of the NAME stack as an inclassion that will push the top item of the INTEGER stack onto the EXEC stack.
        /// </summary>
        IntegerDefine,

        /// <summary>
        /// INTEGER.DUP:
        /// Duplicatelicates the top item on the INTEGER stack. Does not pop its argument (which, if it did, would negate the effect of the duplication!).
        /// </summary>
        IntegerDuplicate,

        /// <summary>
        /// INTEGER.FLUSH:
        /// Empties the INTEGER stack.
        /// </summary>
        IntegerFlush,

        /// <summary>
        /// INTEGER.FROMBOOLEAN:
        /// Pushes 1.0 if the top BOOLEAN is TRUE, or 0.0 if the top BOOLEAN is FALSE.
        /// </summary>
        IntegerFromBoolean,

        /// <summary>
        /// INTEGER.FROMFLOAT:
        /// Pushes the result of truncating the top FLOAT.
        /// </summary>
        IntegerFromFloat,

        /// <summary>
        /// INTEGER.MAX:
        /// Pushes the maximum of the top two items.
        /// </summary>
        IntegerMax,

        /// <summary>
        /// INTEGER.MIN:
        /// Pushes the minimum of the top two items.
        /// </summary>
        IntegerMin,

        /// <summary>
        /// INTEGER.POP:
        /// Pops the INTEGER stack.
        /// </summary>
        IntegerPop,

        /// <summary>
        /// Pushes a integer literal to the INTEGER stack
        /// </summary>
        IntegerPush,

        /// <summary>
        /// INTEGER.RAND:
        /// Pushes a newly generated random INTEGER that is greater than or equal to MIN-RANDOM-INTEGER and less than or equal to MAX-RANDOM-INTEGER.
        /// </summary>
        IntegerRand,

        /// <summary>
        /// INTEGER.ROT:
        /// Rotates the top three items on the INTEGER stack, pulling the third item out and pushing it on top. This is equivalent to "2 INTEGER.YANK".
        /// </summary>
        IntegerRot,

        /// <summary>
        /// INTEGER.SHOVE:
        /// Inserts the top INTEGER "deep" in the stack, at the position indexed by the top INTEGER.
        /// </summary>
        IntegerShove,

        /// <summary>
        /// INTEGER.STACKDEPTH:
        /// Pushes the stack depth onto the INTEGER stack.
        /// </summary>
        IntegerStackDepth,

        /// <summary>
        /// INTEGER.SWAP:
        /// Swaps the top two BOOLEANs.
        /// </summary>
        IntegerSwap,

        /// <summary>
        /// INTEGER.YANK:
        /// Removes an indexed item from "deep" in the stack and pushes it on top of the stack. The index is taken from the INTEGER stack.
        /// </summary>
        IntegerYank,

        /// <summary>
        /// INTEGER.YANKDUP:
        /// Pushes a copy of an indexed item "deep" in the stack onto the top of the stack, without removing the deep item. The index is taken from the INTEGER stack.
        /// </summary>
        IntegerYankDuplicate,

        #endregion Integer

        #region Name

        NamePush,

        /// <summary>
        /// Pushes a floating point value to the FLOAT stack.
        /// </summary>
        NameDefineXExec,

        /// <summary>
        /// NAME.=:
        /// Pushes TRUE if the top two NAMEs are equal, or FALSE otherwise.
        /// </summary>
        NameEquals,

        /// <summary>
        /// NAME.DUP:
        /// Duplicatelicates the top item on the NAME stack. Does not pop its argument (which, if it did, would negate the effect of the duplication!).
        /// </summary>
        NameDuplicate,

        /// <summary>
        /// NAME.FLUSH:
        /// Empties the NAME stack.
        /// </summary>
        NameFlush,

        /// <summary>
        /// NAME.POP: Pops the NAME stack.
        /// </summary>
        NamePop,

        /// <summary>
        /// NAME.QUOTE:
        /// Sets a flag indicating that the next name encountered will be pushed onto the NAME stack (and not have its associated value pushed onto the EXEC stack), regardless of whether or not it has a definition. Upon encountering such a name and pushing it onto the NAME stack the flag will be cleared (whether or not the pushed name had a definition).
        /// </summary>
        NameQuote,

        /// <summary>
        /// NAME.RAND:
        /// Pushes a newly generated random NAME.
        /// </summary>
        NameRand,

        /// <summary>
        /// NAME.RANDBOUNDNAME:
        /// Pushes a randomly selected NAME that already has a definition.
        /// </summary>
        NameRandBoundName,

        /// <summary>
        /// NAME.ROT:
        /// Rotates the top three items on the NAME stack, pulling the third item out and pushing it on top. This is equivalent to "2 NAME.YANK".
        /// </summary>
        NameRot,

        /// <summary>
        /// NAME.SHOVE:
        /// Inserts the top NAME "deep" in the stack, at the position indexed by the top INTEGER.
        /// </summary>
        NameShove,

        /// <summary>
        /// NAME.STACKDEPTH:
        /// Pushes the stack depth onto the INTEGER stack.
        /// </summary>
        NameStackDepth,

        /// <summary>
        /// NAME.SWAP:
        /// Swaps the top two NAMEs.
        /// </summary>
        NameSwap,

        /// <summary>
        /// NAME.YANK:
        /// Removes an indexed item from "deep" in the stack and pushes it on top of the stack. The index is taken from the INTEGER stack.
        /// </summary>
        NameYank,

        /// <summary>
        /// NAME.YANKDUP:
        /// Pushes a copy of an indexed item "deep" in the stack onto the top of the stack, without removing the deep item. The index is taken from the INTEGER stack.
        /// </summary>
        NameYankDuplicate,
        #endregion Name
    }
}